function set_class_first_child(doc, id, c_name) {
    elem = doc.getElementById(id);
    if (elem == null) return null;
    elem = elem.childNodes[0];
    if (elem == null) return null;
    elem.className = c_name;
    return elem;
}
/**
 * Determine if an object or class string contains a given class.
 */
function has_class(o,name) {
    return new RegExp("(^|\\s)"+name+"(\\s|$)").test(o.className);
}
function def(o) {return (typeof o!="undefined");}


function add_class(doc, id, c_name) {
    elem = doc.getElementById(id);
    if (elem == null) return null;
    var c = elem.className || "";
    if (def(c) && !has_class(elem, c_name)) {
	elem.className += (c?" ":"") + c_name;
    }
    return elem;
}

function remove_class_elem(elem, c_name) {
    var c = elem.className || "";
    elem.className = c.replace(new RegExp("(^|\\s)"+c_name+"(\\s|$)"),"$1");
    return elem;
}
function remove_class(doc, id, c_name) {
    elem = doc.getElementById(id);
    if (elem == null) return null;
    var c = elem.className || "";
    elem.className = c.replace(new RegExp("(^|\\s)"+c_name+"(\\s|$)"),"$1");
    return elem;
}

function elementY(element) {
    var t = 0;
    do {
      t += element.offsetTop;
    } while (element = element.offsetParent);
    return t;
}

function load_src_mark_timer() {
    var src_frame = parent.document.getElementsByName("src")[0];
    if (src_frame.loadComplete == false) {
	setTimeout("load_src_mark_timer()", 100);
	return;
    }

    var src_window = src_frame.contentWindow;
    var src_document = src_window.document;
    var src_de = src_document.documentElement;
    var src_line = src_document.getElementById(src_frame.anchor);

    if(src_line != null) {
      var ypos = elementY(src_line) - (src_de.clientHeight - src_line.clientHeight) / 2;
      var xpos = src_window.scrollX;
      var maxy = src_de.scrollHeight - src_de.clientHeight;

      if(ypos < 0) {
        src_window.scroll(xpos, 0);
      } else if(ypos > maxy) {
        src_window.scroll(xpos, maxy);
      } else {
        src_window.scroll(xpos, ypos);
      }
    }

    if (src_document.list != null) {
	for (var i = 0; i < src_document.list.length; i++) {
	    elem = src_document.list[i];
	    if (elem != null)
		elem.className = "line";
	}
    }
    if (src_document.marked_line != null)
	remove_class_elem(src_document.marked_line, "marked");

    var l = [];
    for (var i = 0; i < src_frame.mark.length; i++) {
	var new_class = "mark";
	l[i*3] = set_class_first_child(src_document, src_frame.mark[i], new_class);
	l[i*3+1] = set_class_first_child(src_document, 'V' + src_frame.mark[i], new_class);
	l[i*3+2] = set_class_first_child(src_document, 'S' + src_frame.mark[i], new_class);
    }
    src_document.list = l;
    src_document.marked_line = add_class(src_document, src_frame.anchor, "marked");
}

function load_src_mark(file_id, anchor, s) {
    var src_frame = parent.document.getElementsByName("src")[0];

    if (parent.document.loaded_src == null ||
	parent.document.loaded_src != file_id) {
	src_frame.loadComplete = false;
	src_frame.src = "source_" + file_id + ".html";
	parent.document.loaded_src = file_id;
    }
    src_frame.mark = s;
    src_frame.anchor = anchor;
    setTimeout("load_src_mark_timer()", 10);
}

function load_detail(info_doc, src_file_id, anchor, s, display_source) {
    var info_frame = parent.document.getElementsByName("info")[0];
    info_frame.src = info_doc;
    if (display_source) {
	if (src_file_id != -1)
	    load_src_mark(src_file_id, anchor, s);
	else {
	    var src_frame = parent.document.getElementsByName("src")[0];
	    var src_window = src_frame.contentWindow;
	    var src_document = src_window.document;
	    
	    if (src_document.list != null) {
		for (var i = 0; i < src_document.list.length; i++) {
		    elem = src_document.list[i];
		    if (elem != null)
			elem.className = "line";
		}
	    }
	    
	    if (src_document.marked_line != null)
		remove_class_elem(src_document.marked_line, "marked");
	}
    }
}

function load_issue(number, src_file_id, anchor, s) {
    load_detail('issue-' + number + '.html', src_file_id, anchor, s, true);
}

function load_issue_no_source(number) {
    load_detail('issue-' + number + '.html', -1, 0, false);
}

function load_loop(number, src_file_id, anchor, s) {
    load_detail('loop' + number + '.html', src_file_id, anchor, s, true);
}

function add_expand_section(name) {
    var info_frame = parent.document.getElementsByName("info")[0];
    if (info_frame.expanded_sections == null) {
	info_frame.expanded_sections = {};
    }

    info_frame.expanded_sections[name] = 1;
}

function remove_expand_section(name) {
    var info_frame = parent.document.getElementsByName("info")[0];
    if (info_frame.expanded_sections == null) {
	info_frame.expanded_sections = {};
	return;
    }

    info_frame.expanded_sections[name] = null;
}

function persist_toggle(e) {

    elem = document.getElementById(e);
    if (elem == null) return;
    if (elem.style.display == "none") {
	elem.style.display = "";
	add_expand_section(e);
    } else {
	elem.style.display = "none";
	remove_expand_section(e);
    }
}


function toggle(e) {
    elem = document.getElementById(e);
    if (elem == null) return;
    if (elem.style.display == "none")
	{ elem.style.display = ""; }
    else
	{ elem.style.display = "none"; }
}

function show(e) {
    elem = document.getElementById(e);
    if (elem == null) return;
    elem.style.display = "";
}

function hide(e) {
    elem = document.getElementById(e);
    if (elem == null) return;
    elem.style.display = "none";
}

function show_and_jump(id) {
    show(id + "d");
    show(id + "dminus");
    hide(id + "dplus");

    var document = null;
    var info_frame = parent.document.getElementsByName("info")[0];
    if (info_frame.contentDocument)
	// Mozilla, W3C
	document = info_frame.contentDocument;
    else
	// IE
	document = parent.document.frames["info"].document;

    document.location.hash = id;
}

function statistics(id) {
    elem = document.getElementById(id);
    if (elem == null) return;
    var stat_frame = parent.document.getElementsByName("stat")[0];
    if (stat_frame == null) return;
    var stat_window = stat_frame.contentWindow;
    var stat_document = stat_window.document;
    stat_document.body.innerHTML = elem.innerHTML;
}

function unfold() {
    var info_frame = parent.document.getElementsByName("info")[0];
    if (info_frame.expanded_sections == null) {
	info_frame.expanded_sections = {};
    }
    var a = [];
    for (var i in info_frame.expanded_sections) {
	if (info_frame.expanded_sections[i] == 1) {
	    a.push(i);
	}
    }
    for (var i in a) {
	toggle(a[i]);
	toggle(a[i] + "plus");
	toggle(a[i] + "minus");
    }

    show("sections");
}

var helpwindow = '';

function popitup(url) {
    if (!helpwindow.closed && helpwindow.location) {
	helpwindow.location.href = url;
    } else {
	var w = 640;
	var winW = 630, winH = 460;

	if (parseInt(navigator.appVersion)>3) {
	    if (navigator.appName=="Netscape") {
		winW = top.window.innerWidth;
		winH = top.window.innerHeight;
	    }
	    if (navigator.appName.indexOf("Microsoft")!=-1) {
		winW = top.document.body.offsetWidth;
		winH = top.document.body.offsetHeight;
	    }
	}
	x = winW - w;
	y = 20;
        h = screen.availHeight - 100

	helpwindow = window.open(url, 'help',
				 'height=' + h + ',width=' + w +
				 ',left=' + x + ',top=' + y +
				 ',screenX=' + x + ',screenY=' + y +
				 ',resizable=1,scrollbars=1,toolbar=1');
	helpwindow.moveTo(x, y);
	if (!helpwindow.opener) helpwindow.opener = self;
    }
    if (window.focus) {
	helpwindow.focus();
    }
    return false;
}
