﻿/// <reference path="jquery-1.4.4.js" />
/// <reference path="jquery-ui-1.8.6.custom.min.js" />
/// <reference path="jquery.layout.min-1.2.0.js" />

/*
 * jQuery AjaxQ - AJAX request queueing for jQuery
 *
 * Version: 0.0.1
 * Date: July 22, 2008
 *
 * Copyright (c) 2008 Oleg Podolsky (oleg.podolsky@gmail.com)
 * Licensed under the MIT (MIT-LICENSE.txt) license.
 *
 * http://plugins.jquery.com/project/ajaxq
 * http://code.google.com/p/jquery-ajaxq/
 */

jQuery.ajaxq = function (queue, options)
{
    // Initialize storage for request queues if it's not initialized yet
    if (typeof document.ajaxq == "undefined")
	document.ajaxq = {q:{}, r:null};

    // Initialize current queue if it's not initialized yet
    if (typeof document.ajaxq.q[queue] == "undefined")
	document.ajaxq.q[queue] = [];
    
    if (typeof options != "undefined") {
	// Request settings are given, enqueue the new request

	// Copy the original options, because options.complete is
	// going to be overridden

	var optionsCopy = {};
	for (var o in options) optionsCopy[o] = options[o];
	options = optionsCopy;
	
	// Override the original callback

	var originalCompleteCallback = options.complete;

	options.complete = function (request, status)
	{
	    // Dequeue the current request
	    document.ajaxq.q[queue].shift ();
	    document.ajaxq.r = null;
	    
	    // Run the original callback
	    if (originalCompleteCallback)
		originalCompleteCallback (request, status);

	    // Run the next request from the queue
	    if (document.ajaxq.q[queue].length > 0)
		document.ajaxq.r = jQuery.ajax (document.ajaxq.q[queue][0]);
	};

	// Enqueue the request
	document.ajaxq.q[queue].push (options);

	// Also, if no request is currently running, start it
	if (document.ajaxq.q[queue].length == 1)
	    document.ajaxq.r = jQuery.ajax (options);
    }
    else
    {
	// No request settings are given, stop current request and
	// clear the queue

	if (document.ajaxq.r)
	{
	    document.ajaxq.r.abort ();
	    document.ajaxq.r = null;
	}

	document.ajaxq.q[queue] = [];
    }
}

function get_screen_size() {
    var obj = new Object();
    if (typeof (window.innerWidth) == 'number') {
        //Non-IE
        obj.myWidth = window.innerWidth;
        obj.myHeight = window.innerHeight;
    } else if (document.documentElement && (document.documentElement.clientWidth || document.documentElement.clientHeight)) {
        //IE 6+ in 'standards compliant mode'
        obj.myWidth = document.documentElement.clientWidth;
        obj.myHeight = document.documentElement.clientHeight;
    } else if (document.body && (document.body.clientWidth || document.body.clientHeight)) {
        //IE 4 compatible
        obj.myWidth = document.body.clientWidth;
        obj.myHeight = document.body.clientHeight;
    }
    return obj;
}

function load_url(the_url, selector) {
    $.ajaxq("url", {
	url: the_url,
	cache: false,
	success: function(data) {
	    $(selector).html(data);
	}
    });
    //$.get(url, function (data) { $(selector).html(data); })
}

function sync(f) {
    $.ajaxq("url", {success: f});
}

// Toggle visibility of an item with id 'id'
function toggle(id) {
    $('#' + id).toggle();
}

function persist_toggle(id) {
    toggle(id);
    if (window.expanded_sections == null)
	window.expanded_sections = {};
    window.expanded_sections[id] = $('#' + id).is(":visible");
}

function unfold() {
    if (window.expanded_sections == null)
	window.expanded_sections = {};

    for (var i in window.expanded_sections) {
	if (window.expanded_sections[i]) {
	    $("#" + i).show();
	    $("#" + i + "plus").hide();
	    $("#" + i + "minus").show();
	}
    }
}

function update_tables() {
    document.tablesorter.auto();
    unfold();
}

function stat(id, type, value) {
    load_url("d/stat/" + id + "/" + type + "/" + value, "#sw");
}

function stat_by_kind(id, type) {
    load_url("d/stat_by_kind/" + id + "/" + type, "#sw");
}

function mark_lines(linenumber, highlights) {
    if (linenumber === "") linenumber = null;

    if (window.marked_line != null)
	$('#c #' + window.marked_line).removeClass("mark");
    
    var size = get_screen_size();
    if (linenumber === null)
	$('#c').scrollTo(0,0);
    else {
	if ($('#c #'+linenumber).size() > 0)
	    $('#c').scrollTo('#' + linenumber, 1000, {offset: -size.myHeight/2});
    }
    
    window.marked_line = linenumber;
    if (linenumber != null)
	$('#c #' + window.marked_line).addClass("mark");
    
    if (window.highlights != null) {
	for (var i = 0; i < window.highlights.length; i++) {
	    $('#c #' + window.highlights[i]).removeClass("loop");
	}
    }
    
    window.highlights = highlights;
    for (var i = 0; i < window.highlights.length; i++) {
	$('#c #' + window.highlights[i]).addClass("loop");
    }
}

// Populate a file in the filename pane, mark a line and highlight a
// list of lines
function load_src_mark(file_id, linenumber, highlights) {
    if (file_id === '') {
	load_url("placeholder.html", "#c");
	sync(function() {
	    update_tables();
	    window.file_id = '';
	});
    } else {
	if ( !(window.file_id === file_id)) {
	    load_url("d/source/" + file_id, "#c");
	    window.highlights = null;
	    window.marked_line = null;
	    window.file_id = file_id;
	}
	sync(function() {
	    update_tables();
	    mark_lines(linenumber, highlights);
	});
    }
}

function load_issue_no_source(number) {
    load_url("d/issue/" + number, "#w");
    sync(function() {
	$('#w > div').toggle();
	update_tables();
    });
}

// Load an issue or loop detail in #w, and load a source file in #c,
// highlight it and scroll the line into view.
function load_issue(number, file_id, linenumber, highlights) {
    load_url("d/issue/" + number, "#w");
    sync(function() {
	$('#w > div').toggle();
    });
    load_src_mark(file_id, linenumber, highlights);
}

function load_loop(number, file_id, linenumber, highlights) {
    load_url("d/loop/" + number, "#w");
    sync(function() {
	$('#w > div').toggle();
    });
    load_src_mark(file_id, linenumber, highlights);
}

function show_and_jump(id) {
    $('#' + id + 'd').show();
    $('#' + id + 'dminus').show();
    $('#' + id + 'dplus').hide();
    $('#w').scrollTo('#' + id + 'd', 0, {offset:-30}); // Not optimal, but #id doesn't work for some reason
}

function objsize(obj) {
    var size = 0, key;
    for (key in obj) {
        if (obj.hasOwnProperty(key)) size++;
    }
    return size;
};

function update_error_pane() {
    if (window.error != null) {
	if (objsize(window.error) != 0)
	    window.outerLayout.show("north");
	else
	    window.outerLayout.hide("north");

    } else {
	window.outerLayout.hide("north");
    }
}


function push_error(tag,msg) {
    if (window.error == null)
	window.error = {};
    window.error[tag] = msg;

    var div = $('<div></div>');
    for (var key in window.error) {
	div.append(window.error[key]);
    }
    $('#n').html(div);
    update_error_pane();
}

function pop_error(tag) {
    if (window.error != null)
	delete window.error[tag];
    update_error_pane();
}

function license_error() {
    push_error("license", '<div class="warning"><p>License server communication error. Browsing will not work until the license becomes available.</p></div>');
}

function license_ok() {
    pop_error("license");
}

function ping() {
    $.ajaxq("url",
	    {
		url: "/d/ping",
		cache: false,
		datatype: "script",
		success: function(data) {
		    pop_error("server");
		    setTimeout("ping()", 5000);
		},
		error: function(data) {
		    push_error("server", '<div class="warning"><p>The report server has disconnected. You need to reopen the report.</p></div>');
		    setTimeout("ping()", 5000);
		}
	    }
	   );
}

var helpwindow = '';

function popitup(url) {
    if (!helpwindow.closed && helpwindow.location) {
	helpwindow.location.href = url;
    } else {
	var w = 640;
	var winW = 630, winH = 460;

	if (parseInt(navigator.appVersion)>3) {
	    if (navigator.appName=="Netscape") {
		winW = top.window.innerWidth;
		winH = top.window.innerHeight;
	    }
	    if (navigator.appName.indexOf("Microsoft")!=-1) {
		winW = top.document.body.offsetWidth;
		winH = top.document.body.offsetHeight;
	    }
	}
	x = winW - w;
	y = 20;
        h = screen.availHeight - 100

	helpwindow = window.open(url, 'help',
				 'height=' + h + ',width=' + w +
				 ',left=' + x + ',top=' + y +
				 ',screenX=' + x + ',screenY=' + y +
				 ',resizable=1,scrollbars=1,toolbar=1');
	helpwindow.moveTo(x, y);
	if (!helpwindow.opener) helpwindow.opener = self;
    }
    if (window.focus) {
	helpwindow.focus();
    }
    return false;
}

function load_contents(initial_contents, after) {
    if (location.hash == "" || location.hash == "#") {
        initial_contents();
    } else {
        $.ajax({
            url: location.hash.substr(1) + ".js",
            dataType: 'script',
            error: function (xhr, status, e) {
                initial_contents();
                location.hash = "";
            }
        });
    }

    $.ajax({
	url: "d/title",
	dataType: 'script'
    });

    sync(after);
    ping();
}
